/*
   * pitchtune - an instrument tuner
   * Copyright (C) 2000 Haakon Andre Hjortland
   * 
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License
   * as published by the Free Software Foundation; either version 2
   * of the License, or (at your option) any later version.
   * 
   * This program is distributed in the hope that it will be useful,
   * but WITHOUT ANY WARRANTY; without even the implied warranty of
   * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   * GNU General Public License for more details.
   * 
   * You should have received a copy of the GNU General Public License
   * along with this program; if not, write to the Free Software
   * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   * 
   */

/**
 * \file main.h
 * \brief Main header file
 */


/**
 * \page Concepts
 * \section Definitions
 * \subsection Notes
 * a^1 is considered note=0
 * 
 * c^1 - h^1 is considered octave=0
 *
 * \subsection Samplepieces
 * Name of the wave-pieces displayed
 */



/****/

/* Prevent this file from being included twice */
#ifndef _MAIN_H
#define _MAIN_H



/*** Includes */

/* Contains data type definitions */
#include <stdlib.h>
#include <sys/types.h> /* for u_int8_t */
#include <gtk/gtk.h>

#include "glade_callbacks.h"
#include "glade_interface.h"
#include "glade_support.h"
#include "sel_guitar.h"
#include "tuner.h"
#include "sound.h"
#include "control.h"

/*** Macros */

#define DRAWAREA_WIDTH_QUARTER  100
#define DRAWAREA_WIDTH          DRAWAREA_WIDTH_QUARTER*4
#define DRAWAREA_HEIGHT_QUARTER 64
#define DRAWAREA_HEIGHT         DRAWAREA_HEIGHT_QUARTER*4

#define SAMPLEPIECES_MAX 100

#define SOUNDBUFFER_SIZE 100000

#define FREQUENCY_MIN 10
#define FREQUENCY_MAX 10000

#define NOTE_MIN -100
#define NOTE_MAX 100

/* 12th root of 2, ratio between frequency of two adjacent semi-tones */
#define RT12_2 1.059463094

/* Print debug information */
#define PRINTF_DEBUG(format, args...) \
    { \
    GString *printf_debug_gstr1=g_string_new(""); \
    g_string_sprintf(printf_debug_gstr1, format, ##args); \
    print_to_debug_window(printf_debug_gstr1->str); \
    g_string_free(printf_debug_gstr1, TRUE); \
    }

/* Scale note strings */
#define SCALE_COUNT 2
const char *(note_chars_lowercase_multi[2][12]);
const char *(note_chars_uppercase_multi[2][12]);

/* Menu entry names */
#define MENU_ENTRIES 6
const char *menu_entry[MENU_ENTRIES];




struct config_struct_window
{
    int xpos;
    int ypos;
    gboolean pos_set;
};



/**** Global variables */




/* GTK variables */

/* Main window */
GtkWidget *window_main;
GtkWidget *drawingarea1;
GtkWidget *spinbutton_freq;
GtkWidget *spinbutton_a1freq;
GtkWidget *optionmenu_noteselect;
GtkWidget *optionmenu_noteselect_menu;
GtkWidget *checkbutton_ministep_freq;
GtkWidget *checkbutton_ministep_a1freq;
GtkWidget *checkbutton_coarse_tuning;
GtkWidget *hscale_finetune;
GtkWidget *hscale_coarsetune;
GtkWidget *label_freqwarn;
GtkWidget *mmenu_item_sel_guitar;
GtkWidget *mmenu_item_small_display;
GtkWidget *button_muting1;
GtkAdjustment *adjustment_finetune;
GtkAdjustment *adjustment_coarsetune;
GdkPixmap *gdkpixmap1;

/* Settings window */
GtkWidget *window_settings;
GtkWidget *settings_entry_sounddev;
GtkWidget *settings_alsa_device;
GtkWidget *settings_radiobutton_scale_1;
GtkWidget *settings_radiobutton_scale_2;

/* Filesel window */
GtkWidget *fileselection1;

/* About dialog */
GtkWidget *about_dialog;

/* Help dialog */
GtkWidget *help_dialog;

/* Debug window */
GtkWidget *debug_window;
GtkWidget *debug_text_box;

/* GdkImage containing the drawn waves */
GdkImage *gdkimage1;
/* GdkVisual for drawingarea1 */
GdkVisual *gdkvisual1;

/* Graphics Contexts */
GdkGC *GdkGC_black;
GdkGC *GdkGC_red;

/* Configuration structures */
struct
{
    struct config_struct_window window;
} sel_guitar_config;

struct
{
    struct config_struct_window window;
} main_config;

/**
 * \brief Holds configuration from reading config file to realizing the configuration
 */
struct
{
    gboolean sel_guitar_visible;
    double a1frequency;
    double frequency;
    gboolean note_set;
    gboolean note;
    int decay_coarse;
    int decay_fine;
    gboolean display_big;
    int menu_accel_key[MENU_ENTRIES];
    int menu_accel_mods[MENU_ENTRIES];
    double play_sine;
} config;

/* --- */
int tuner_started; /**< Has the tuner been started? */

/* Sound buffer */
u_int8_t soundbuffer[SOUNDBUFFER_SIZE]; /* Rotating buffer */
int soundbuffer_len;
int soundbuffer_readpos;
int soundbuffer_writepos;
double backto_lastdraw_samplecount; /* Samples from writepos */
/* to the sample which was sample 1 at last draw */
/* This variable will be negative */

/* Samplepieces buffer */
u_int8_t samplepieces_buffer[DRAWAREA_WIDTH*SAMPLEPIECES_MAX];
/* Rotating buffer */
int samplepieces_which_is_zero; /**< Which slot is number 0 */
/**
 * \brief Number of times to draw over and over a samplepiece
 */
int samplepieces_count; /* How many slots to use */
/**
 * \brief Number of samplepieces to stack on display in finetune mode
 */
int samplepieces_count_finetune;
/**
 * \brief Number of samplepieces to stack on display in coarsetune mode
 */
int samplepieces_count_coarsetune;
int samplepieces_last_valid; /**< How many valid slots -1 */

/* --- */
double frequency; /**< Frequency */
gboolean frequency_set;
double wavelength_samples; /**< Wavelength in samples */
int samplerate; /**< Sampling rate */
double downsample_ratio; /**< Downsample ratio */

int tuner_mode_coarse; /**< If tuner is in coarse mode */
int tuner_mode_coarse_new; /**< If tuner should be put in coarse mode */

gboolean notes_note_set;
int notes_note;

int timer_tag; /**< For GTK timeout */
int timer_count; /**< For coarse tuning. Update only every n timer tick. */

int sound_fd; /**< File descritor for sound input */

int spinbutton_freq_changed_by_user; /**< Set to FALSE before program changes frequency. */

gboolean displaystate_is_splash; /**< Showing splash-screen? */

gboolean display_big; /**< Big or small display? */
gboolean display_cleared_around;

/* Standard selector */
int sel_standard_note;
int sel_standard_octave;
int sel_standard_last_octave; /* Last octave-menu created */
gboolean sel_standard_last_octave_valid;
int sel_standard_last_scale_type;

/*--- */
GString *gstr_sounddev;
GString *gstr_alsa_device;
int scale_type; /*Which type of scale to use */



/**** Function declarations */

/* in main.c */
void config_load();
void config_save();
double double_modulus(double a, double b);
void exit_program();
void print_to_debug_window(char *str);

/* unused:
void main_init();
void create_samplepieces_colormap(int color_count);
void set_samplepieces_count(int number);
gboolean func_drawingarea1_expose (GtkWidget *widget,
				   GdkEventExpose *event, gpointer user_data); 
void func_hscale1_value_changed (GtkAdjustment *adjustment, gpointer user_data);
*/


/* in ui.c */
void check_redo_display();
void sel_standard_set_note(gboolean note_set, int note);
void on_adjustment_finetune_value_changed (GtkAdjustment *adjustment,
					   gpointer user_data);
void on_adjustment_coarsetune_value_changed (GtkAdjustment *adjustment,
					     gpointer user_data);
void on_optionmenu_noteselect_menu_deactivate(GtkMenuShell *menu_shell,
					      gpointer data);

void sel_standard_update(gboolean inform_set_note);
void sel_standard_inform_set_note();
void sel_guitar_set_active(gboolean active);
void on_button_set_a1_default_clicked(GtkButton *button, gpointer user_data);
void on_button_stop_clicked(GtkButton *button, gpointer user_data);
void on_button_start_clicked (GtkButton *button, gpointer user_data);
gboolean on_window_main_delete_event (GtkWidget *widget, GdkEvent *event,
				      gpointer user_data);
gboolean on_drawingarea1_expose_event (GtkWidget *widget,
				       GdkEventExpose *event, gpointer user_data);
void on_button_octaveup_clicked (GtkButton *button, gpointer user_data);
void on_button_octavedown_clicked (GtkButton *button, gpointer user_data);
void on_spinbutton_freq_changed (GtkEditable *editable, gpointer user_data);
void on_spinbutton_a1freq_changed (GtkEditable *editable, gpointer user_data);
void on_checkbutton_ministep_freq_toggled (GtkToggleButton *togglebutton,
					   gpointer user_data);
void on_checkbutton_ministep_a1freq_toggled (GtkToggleButton *togglebutton,
					     gpointer user_data);
void on_checkbutton_coarse_tuning_toggled (GtkToggleButton *togglebutton,
					   gpointer user_data);

/* Main menu */
void on_mmenu_item_settings_activate (GtkMenuItem *menuitem,
				      gpointer user_data);
void on_mmenu_item_exit_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_mmenu_item_sel_guitar_toggled (GtkCheckMenuItem *checkmenuitem,
				       gpointer user_data);
void on_mmenu_item_about_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_mmenu_item_help_activate (GtkMenuItem *menuitem, gpointer user_data);
void on_mmenu_item_small_display_toggled (GtkCheckMenuItem *checkmenuitem,
					  gpointer user_data);

/* Settings */
void on_settings_button_ok_clicked (GtkButton *button, gpointer user_data);
void on_settings_button_cancel_clicked (GtkButton *button, gpointer user_data);
void on_settings_button_browse_sounddev_clicked (GtkButton *button,
						 gpointer user_data);
void on_window_settings_destroy (GtkObject *object, gpointer user_data);
void on_fileselection1_ok_clicked (GtkFileSelection *selector,
				   gpointer user_data);
void on_fileselection1_cancel_clicked (GtkFileSelection *selector,
				       gpointer user_data);
void on_fileselection1_destroy (GtkObject *object, gpointer user_data);

/* About dialog */
void on_about_dialog_button_ok_clicked (GtkButton *button, gpointer user_data);

/* Help dialog */
void on_help_dialog_button_ok_clicked (GtkButton *button, gpointer user_data);

/* Debug window */
gboolean on_debug_window_delete_event (GtkWidget *widget, GdkEvent *event,
				       gpointer user_data);



/* #ifndef _MAIN_H */
#endif


