/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * Copyright (c) 2005 Alexandre Bourget
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file sound.c
 * \brief Sound data input and output functions
 *
 */



/*** Includes */

#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <errno.h>
/* for M_PI  */
#include <math.h>
#define M_PI            3.14159265358979323846  /* pi */

#define USE_ALSA


#ifdef USE_ALSA
/* ALSA support */
#include <alsa/asoundlib.h>
#else /* use OSS */
/* OSS support - DEPRECATED */
#include <sys/soundcard.h>
#endif /* end USE_ALSA */


#include "main.h"



/* ############################ ALSA VERSION ################################ */

#ifdef USE_ALSA

/* Global stuff */
snd_pcm_hw_params_t *hwparams, *pl_hwparams;        /**< Hardware parameters */
snd_pcm_sw_params_t *swparams, *pl_swparams;        /**< Software parameters */
signed short *samples;                              /**< Sample data */
snd_pcm_channel_area_t *areas;                      /**< Channel areas */
snd_pcm_t *rec_pcm = NULL;                          /**< PCM input stream */
snd_pcm_t *play_pcm = NULL;                         /**< PCM playback stream */
snd_output_t *output = NULL;                        /**< Output object */
snd_pcm_sframes_t buffer_size;                      /**< Ring buffer size */
snd_pcm_sframes_t period_size;                      /**< Period size */
snd_pcm_format_t format = SND_PCM_FORMAT_U8;        /**< sample format, to preserve compat. with OSS version */
snd_pcm_format_t play_format = SND_PCM_FORMAT_S16;  /**< sample format for sine playback */
unsigned int rate = 44100;                          /**< Sample rate for capture (unused?!?) */
unsigned int channels = 1;                          /**< Channels count (mono) */
unsigned int buffer_time = 500000;                  /**< ring buffer length in microsecs */
unsigned int period_time = 100000;                  /**< period time in microsecs */
char errmsg[255] = {0};                             /**< Error message buffer for SND_CHECK_ERROR() */
double sine_step = 0;                               /**< Step between frames (2pi * freq / rate) */
double sine_freq = 1000;                            /**< Sine wave frequency */
double phase = 0;                                   /**< Where we at in the sin() func (sine_step * sample number) */
/*double play_sine = 0;                               / **< Sine wave playback: 0 = muted, 1 = unmuted, default: muted*/

/**
 * \brief Shortcut for error checking when running snd_pcm_* functions
 */
#define SND_CHECK_ERROR(val_x, val_y) err = val_x; if (err < 0) { strcpy(errmsg, val_y); printf(strcat(errmsg, ": %s\n"), snd_strerror(err)); return err; }

/**
 * \brief Shortcut for error checking when running snd_pcm_* functions, with a parameter.
 */
#define SND_CHECK_ERROR_PARM(val_x, val_y, val_z) \
err = val_x; \
if (err < 0) { strcpy(errmsg, val_y); printf(strcat(errmsg, ": %s\n"), val_z, snd_strerror(err)); return err; }



/**
 * \brief Initialize sound device for capture and playback
 *
 * \return Null on success, non-null on error
 */
int sound_init ()
{
    int err;


    snd_pcm_hw_params_alloca(&hwparams);
    snd_pcm_sw_params_alloca(&swparams);

    snd_pcm_hw_params_alloca(&pl_hwparams);
    snd_pcm_sw_params_alloca(&pl_swparams);

    SND_CHECK_ERROR(snd_output_stdio_attach(&output, stdout, 0),
                    "Output failed");

    sound_capture_init();
    sound_playback_init();


    return(0);
}




/**
 * \brief Init the capture PCM device that will record sound to be analyzed and displayed.
 * 
 * \return Null on success, non-nil otherwise.
 */
int sound_capture_init()
{
    int err, dir;


    /* open device for capture */
    SND_CHECK_ERROR(snd_pcm_open(&rec_pcm, gstr_alsa_device->str, SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK),
                    "Capture open error");

    /* Set HW params */

    SND_CHECK_ERROR(snd_pcm_hw_params_any(rec_pcm, hwparams),
                    "Broken configuration for capture: no configurations available");

    /* set the interleaved read/write format */
    SND_CHECK_ERROR(snd_pcm_hw_params_set_access(rec_pcm, hwparams, SND_PCM_ACCESS_RW_INTERLEAVED),
                    "Access type not available for capture");

    /* set the stream rate */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_rate_near(rec_pcm, hwparams, &rate, 0),
                         "Rate %iHz not available for capturing: %s\n", rate);

    /* set for tuner.c */
    samplerate = rate;
    /* set the sample format */
    format = SND_PCM_FORMAT_U8; /* to preserve compatibility with the old OSS version */

    SND_CHECK_ERROR(snd_pcm_hw_params_set_format(rec_pcm, hwparams, format),
                    "Sample format not available for capture");

    /* set the count of channels: 1 = mono, hardcoded in the heading of this file */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_channels(rec_pcm, hwparams, channels),
                         "Channels count (%i) not available for capture", channels);

    /* set the buffer time */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_buffer_time_near(rec_pcm, hwparams, &buffer_time, &dir),
                         "Unable to set buffer time %i for capture", buffer_time);


    SND_CHECK_ERROR(snd_pcm_hw_params_get_buffer_size(hwparams, &buffer_size),
                    "Unable to get buffer size for capture");

    /* set the period time */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_period_time_near(rec_pcm, hwparams, &period_time, &dir),
                         "Unable to set period time %i for capture", period_time);

    SND_CHECK_ERROR(snd_pcm_hw_params_get_period_size(hwparams, &period_size, &dir),
                    "Unable to get period size for capture");
    /* write the parameters to device */
    SND_CHECK_ERROR(snd_pcm_hw_params(rec_pcm, hwparams),
                    "Unable to set hw params for capture");



    /* Set SW params */



    /* get the current swparams */
    SND_CHECK_ERROR(snd_pcm_sw_params_current(rec_pcm, swparams),
                    "Unable to determine current swparams for capture");

    /* start the transfer when the buffer is almost full: */
    /* (buffer_size / avail_min) * avail_min */
    /*    SND_CHECK_ERROR(snd_pcm_sw_params_set_start_threshold(rec_pcm, swparams, (buffer_size / period_size) * period_size), */
    /*                    "Unable to set start threshold mode for capture"); */

    /* allow the transfer when at least period_size samples can be processed */
    /*    SND_CHECK_ERROR(snd_pcm_sw_params_set_avail_min(rec_pcm, swparams, period_size), */
    /*                    "Unable to set avail min for capture"); */

    /* align all transfers to 1 sample */
    SND_CHECK_ERROR(snd_pcm_sw_params_set_xfer_align(rec_pcm, swparams, 1),
                    "Unable to set transfer align for capture");

    /* write the parameters to the capture device */
    SND_CHECK_ERROR(snd_pcm_sw_params(rec_pcm, swparams),
                    "Unable to set sw params for capture");


#ifdef DEBUG
/*     snd_pcm_dump(rec_pcm, output); */
/*     printf("\n\ndump_hw_setup\n\n"); */
/*     snd_pcm_dump_hw_setup(rec_pcm, output); */
/*     printf("\n\ndump_sw_setup\n\n"); */
/*     snd_pcm_dump_sw_setup(rec_pcm, output); */
/*     printf("\n\ndump_setup\n\n"); */
/*     snd_pcm_dump_setup(rec_pcm, output); */
#endif


    return 0;
}



/**
 * \brief Init the playback PCM device that will play the current frequency
 * 
 * \return Null on success, otherwise non-zero.
 */
int sound_playback_init()
{
    int err, dir;


    /* open device for playback */
    SND_CHECK_ERROR(snd_pcm_open(&play_pcm, gstr_alsa_device->str, SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK),
                    "Playback open error");

    /* set HARDWARE parameters */

    /* choose all parameters */
    SND_CHECK_ERROR(snd_pcm_hw_params_any(play_pcm, pl_hwparams),
                    "Broken configuration for playback: no configuration available");
                    /* that means that the hardware configuration is broken on the system,
                       because coherent configuration can be found */


    /* set access mode (MMAP, read/write, etc..) */
    SND_CHECK_ERROR(snd_pcm_hw_params_set_access(play_pcm, pl_hwparams, SND_PCM_ACCESS_RW_INTERLEAVED),
                    "Access type not available for playback");

    /* set the sample format SND_PCM_FORMAT_S16, see header */
    SND_CHECK_ERROR(snd_pcm_hw_params_set_format(play_pcm, pl_hwparams, play_format),
                    "Sample format not available for playback");

    /* set channels to mono */
    SND_CHECK_ERROR(snd_pcm_hw_params_set_channels(play_pcm, pl_hwparams, 1),
                    "Mono mode not available for playback");

    /* set sample rate */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_rate(play_pcm, pl_hwparams, rate, 0),
                         "Can't select sample rate %d", rate);


    /* set buffer time */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_buffer_time_near(play_pcm, pl_hwparams, &buffer_time, &dir),
                         "Unable to set buffer time %i for playback", buffer_time);
    SND_CHECK_ERROR(snd_pcm_hw_params_get_buffer_size(pl_hwparams, &buffer_size),
                    "Unable to get buffer size for playback");

    /* set the period time */
    SND_CHECK_ERROR_PARM(snd_pcm_hw_params_set_period_time_near(play_pcm, pl_hwparams, &period_time, &dir),
                         "Unable to set period time %i for playback", period_time);
    SND_CHECK_ERROR(snd_pcm_hw_params_get_period_size(pl_hwparams, &period_size, &dir),
                    "Unable to get period size for playback");

    /* write params to device */
    SND_CHECK_ERROR(snd_pcm_hw_params(play_pcm, pl_hwparams),
                    "Unable to set hw params for playback");


    /* set SOFTWARE parameters */
    /* get the current swparams */
    SND_CHECK_ERROR(snd_pcm_sw_params_current(play_pcm, pl_swparams),
                    "Unable to determine current swparams for playback");

    /* align all transfers to 1 sample */
    SND_CHECK_ERROR(snd_pcm_sw_params_set_xfer_align(play_pcm, pl_swparams, 1),
                    "Unable to set transfer align for playback");
             
    /* allow the transfer when at least period_size samples can be processed */
    SND_CHECK_ERROR(snd_pcm_sw_params_set_avail_min(play_pcm, pl_swparams, period_size),
                    "Unable to set avail min for playback");

#ifdef DEBUG
    snd_pcm_dump(play_pcm, output);
    printf("\n\ndump_hw_setup\n\n");
    snd_pcm_dump_hw_setup(play_pcm, output);
    printf("\n\ndump_sw_setup\n\n");
    snd_pcm_dump_sw_setup(play_pcm, output);
    printf("\n\ndump_setup\n\n");
    snd_pcm_dump_setup(play_pcm, output);
#endif





    return 0;
}



/**
 * \brief Closes capture and playback sound devices
 *
 * \return Null
 */
int sound_deinit()
{
    int err;

#ifdef DEBUG
    printf ("Closing capture device... ");
#endif
    if (rec_pcm) {
        SND_CHECK_ERROR(snd_pcm_close(rec_pcm),
                        "Error closing ALSA capture device");

#ifdef DEBUG        
        printf("capture closed\n");
#endif
        rec_pcm = NULL; /* This is closed now */
    } else {
#ifdef DEBUG
        printf("capture device not opened\n");
#endif
    }

#ifdef DEBUG
    printf ("Closing playback device... ");
#endif
    if (play_pcm) {
        SND_CHECK_ERROR(snd_pcm_close(play_pcm),
                        "Error closing ALSA playback device");

#ifdef DEBUG
        printf("playback closed\n");
#endif
        play_pcm = NULL; /* This is closed now */
    } else {
#ifdef DEBUG
        printf("playback device not opened\n");
#endif
    }

    return 0;
}


/**
 * \brief Calls sound reader and sound writer for capture and playback
 * \return Non-nil on error
 *
 * This is just a quick call for sound_readCaptureStream() and sound_writePlaybackStream()
 */
int sound_readwrite()
{
    int err = 0;
  
    /* Just to differentiate errors from read_capture and write_playback */
    err += 1   * sound_read_capture_stream();
    err += 100 * sound_write_playback_stream();

    return err;
}






/**
 * \brief Pumps data from sound card to buffer, and gen de sine wave for playback.
 * \return Null on success, non-null otherwise
 */
int sound_read_capture_stream()
{
    int int1;
    int int2;
    int int3;
    int writepos_0;
    u_int8_t chars[8];


    int2 = soundbuffer_len;

    soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
    soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
    if (soundbuffer_writepos < soundbuffer_readpos)
	{
	    soundbuffer_writepos += SOUNDBUFFER_SIZE;
	}

    writepos_0=soundbuffer_writepos;

  label0:
    int1 = snd_pcm_readi(rec_pcm, chars, 8);
/* read (sound_fd, &(chars[0]), 8); // Q: Why read 8 bytes? A: Seems some */
    /* systems have trouble feeding us */
    /* bytes one by one. (Could be a */
    /* format / channels problem.) */
    if ((int1 >= 1) && (int1 <= 8))  /* from 1 to 8 */
	{
          /* Got some data */
	    for (int3 = 0; int3 < int1; int3++)
		soundbuffer[soundbuffer_writepos++ % SOUNDBUFFER_SIZE] = chars[int3];
	    
	    goto label0;
	}
    else if (int1 == 0)
	{
          /* Nothing read */
	}
    else
	{
	    if (int1 == -EAGAIN) {
              /* ok.. in NON_BLOCK mode */
	    }
	    else if (int1 == -EBADFD) {
		printf("PCM in bad state: %s\n", snd_strerror(int1));
	    }
	    else if (int1 == -EPIPE) {
		printf("Overrun occured: %s\n", snd_strerror(int1));
	    }
	    else if (int1 == -ESTRPIPE) {
		printf("A suspend event occurred (stream is suspended and waiting for an application recovery): %s\n", snd_strerror(int1));
	    }
	    else {
		printf("An error occurred while reading: %s\n", snd_strerror(int1));
		printf("SYSTEM error int1: %d erro: %d: %s\n", int1, errno, strerror(errno));
	    }
	    /*
              if (!(errno == EAGAIN)) // Nonblocking is not an error
		{
		    PRINTF_DEBUG("Error reading from sound device (Debug info:%d, %s)\n",
				 int1, strerror(errno));
		}
	    */
	}

    /* Some samples in the buffer might have been discarded, */
    /* so update soundbuffer_readpos */
    if ( (soundbuffer_writepos - soundbuffer_readpos) >= SOUNDBUFFER_SIZE )
	{
	    soundbuffer_readpos = soundbuffer_writepos + 1;
	}

    /* Update backto_lastdraw_samplecount */
    backto_lastdraw_samplecount-=(soundbuffer_writepos - writepos_0);

    /* Update soundbuffer_len */
    soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
    soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
    if (soundbuffer_writepos < soundbuffer_readpos)
	{
	    soundbuffer_writepos += SOUNDBUFFER_SIZE;
	}
    soundbuffer_len = soundbuffer_writepos - soundbuffer_readpos;

    if (soundbuffer_len < int2) PRINTF_DEBUG("soundbuffer_len < int2");

    return(0);

}




/**
 * \brief Function that will generate the sine wave required for note playback
 *
 * \param phase Position between 0 and 2pi where we start to fill it
 * \param sine_buffer Buffer of u_int16 for sinewave
 * \param frames Number of frames to fill
 *
 * \return Null on success, error code otherwise
 */
int generate_sine_wave(double phase, unsigned short *sine_buffer, int frames)
{
    double res_double;
    int i;
    
    for (i = 0; i < frames; i++) {
        res_double = sin(phase) * 32767 * config.play_sine; /* mute = 0 */
        /*printf("My double num: %f\n", res_double); */
        sine_buffer[i] = (unsigned short) res_double;
        phase += sine_step;
    }

    return 0;
}



/**
 * \brief Change muting state
 *
 * \param muting Boolean, 0 = muted, 1 = unmuted
 *
 * \return Null on success, error code otherwise
 */
int sound_muting(int muting)
{
    config.play_sine = muting;

    return 0;
}

/**
 * \brief Change frequency of sound
 *
 * \param freq New frequency for playback
 * 
 * \return Null on success, non-null otherwise
 */
int sound_change_freq(double freq)
{
    sine_freq = freq;
    sine_step = 2 * M_PI * freq / 44100;

    return 0;
}


/**
 * \brief Writes sine wave to playback device
 *
 * \return Non-null on error, 0 on success
 */
int sound_write_playback_stream()
{
    /*int err; // for PCM_CHECK_ERROR */
    unsigned short sine_buffer[SOUNDBUFFER_SIZE]; /* Buffer for sine wave samples */
    snd_pcm_sframes_t written_frames;
    double max_phase = 2 * M_PI;

    if (!sine_step)
        sound_change_freq(1000);

    generate_sine_wave(phase, sine_buffer, SOUNDBUFFER_SIZE);

    written_frames = snd_pcm_writei(play_pcm, sine_buffer, (snd_pcm_uframes_t) SOUNDBUFFER_SIZE);

    /* Error ? */
    if (written_frames < 0) {
        switch(written_frames)
            {
            case -EAGAIN:
                /* non-blocking: when nothing to read */
                /* that's all right */
                break;
            case -EBADFD:
                /* oh my!! */
                printf("%s:%d - BADFD\n", __FILE__, __LINE__);
                break;
            case -EPIPE:
                printf("%s:%d - EPIPE\n", __FILE__, __LINE__);
                break;
            case -ESTRPIPE:
                printf("%s:%d - ESTRPIPE\n", __FILE__, __LINE__);
                break;
            default:
                printf("%s:%d - What ?\n", __FILE__, __LINE__);
                break;
            }        
    }
    else {
        /* Advance phase from the right number of frames we wrote */
        phase += (double) written_frames * sine_step;

        /* Modulo operator emulator, since % doesn't work here */
        while(phase > max_phase)
            phase -= max_phase;
    }


    return 0;
}

























#else /* OSS version */

/* ############################# OSS VERSION ################################# */



















/*
 * Unused anyway in OSS version
 */
int sound_change_freq(double freq)
{
    return 0;
}

/*
 * Unused anyway in OSS version
 */
int sound_muting(int muting)
{
    return 0;
}

/*
 * sound_init
 *
 * Return 0 on success
 *
 */

int sound_init ()
{
    int int1;
    int format;
    int channels;

    /* Open sound device */
    sound_fd = open (gstr_sounddev->str, O_RDONLY);
    if (sound_fd <= 0)
	{
	    PRINTF_DEBUG("Error opening %s: %s\n", gstr_sounddev->str, strerror(errno));
	    return (-1);
	}

    /* Set format */
    format = AFMT_U8;
    int1 = ioctl(sound_fd, SNDCTL_DSP_SETFMT, &format);
    if (int1 != 0)
	{
	    PRINTF_DEBUG("Could not set audio format\n");
	    close(sound_fd);
	    return (-1);
	}

    /* Set number of channels */
    channels = 1;
    int1 = ioctl(sound_fd, SNDCTL_DSP_CHANNELS, &channels);
    if (int1 != 0)
	{
	    PRINTF_DEBUG("Could not set number of channels\n");
	    close(sound_fd);
	    return (-1);
	}

    /* Set samplerate */
    samplerate = 44100;
    int1 = ioctl(sound_fd, SNDCTL_DSP_SPEED, &samplerate);
    if (int1 != 0)
	{
	    PRINTF_DEBUG("Could not set samplerate\n");
	    close(sound_fd);
	    return (-1);
	}

    /* Check if the parameters were set correctly */
    if (format != AFMT_U8)
	{
	    PRINTF_DEBUG("Desired audio format not supported\n");
	    close(sound_fd);
	    return (-1);
	}

    if (channels != 1)
	{
	    PRINTF_DEBUG("Could not set recording to mono\n");
	    close(sound_fd);
	    return (-1);
	}

    if ((samplerate > 1000000) || (samplerate < 1000))
	{
	    PRINTF_DEBUG("Samplerate is %d. Can't accept that.\n", samplerate);
	    close(sound_fd);
	    return (-1);
	}

    int1=fcntl(sound_fd, F_SETFL, O_NONBLOCK);
    if (int1 == -1)
	{
	    PRINTF_DEBUG("Error setting %s to nonblocking mode\n", gstr_sounddev->str);
	    close(sound_fd);
	    return (-1);
	}

    return(0);
}



/*
 * sound_deinit
 *
 */

int sound_deinit()
{
    int int1;

    if (sound_fd >= 0)
	{
	    int1=close(sound_fd);
	    if (int1 != 0)
		PRINTF_DEBUG("Error closing sound device\n");
	    sound_fd=-1;
	}

    return;
}


/*
 * Just call the other readtobuffer func, for compat. with ALSA version
 */
int sound_readwrite()
{
    return sound_readtobuffer();
}


/*
 * sound_readtobuffer
 *
 * Returns 0 on success
 *
 */
int sound_readtobuffer ()
{
    int int1;
    int int2;
    int int3;
    int writepos_0;
    u_int8_t chars[8];


    int2 = soundbuffer_len;

    soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
    soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
    if (soundbuffer_writepos < soundbuffer_readpos)
	{
	    soundbuffer_writepos += SOUNDBUFFER_SIZE;
	}

    writepos_0=soundbuffer_writepos;

  label0:
    int1 = read (sound_fd, &(chars[0]), 8); /* Q: Why read 8 bytes? A: Seems some */
    /* systems have trouble feeding us */
    /* bytes one by one. (Could be a */
    /* format / channels problem.) */
    if ((int1 >= 1) && (int1 <= 8)) /* from 1 to 8 included */
	{
	    /* Got some data */
	    for (int3 = 0; int3 < int1; int3++)
		soundbuffer[soundbuffer_writepos++ % SOUNDBUFFER_SIZE] = chars[int3];
    
	    goto label0;
	}
    else if (int1 == 0)
	{
	    /* Nothing read */
	}
    else
	{
	    if (!(errno == EAGAIN)) /* Nonblocking is not an error */
		{
		    PRINTF_DEBUG("Error reading from sound device (Debug info:%d, %s)\n",
				 int1, strerror(errno));
		}
	}

    /* Some samples in the buffer might have been discarded, */
    /* so update soundbuffer_readpos */
    if ( (soundbuffer_writepos - soundbuffer_readpos) >= SOUNDBUFFER_SIZE )
	{
	    soundbuffer_readpos = soundbuffer_writepos + 1;
	}

    /* Update backto_lastdraw_samplecount */
    backto_lastdraw_samplecount-=(soundbuffer_writepos - writepos_0);

    /* Update soundbuffer_len */
    soundbuffer_readpos   = soundbuffer_readpos  % SOUNDBUFFER_SIZE;
    soundbuffer_writepos  = soundbuffer_writepos % SOUNDBUFFER_SIZE;
    if (soundbuffer_writepos < soundbuffer_readpos)
	{
	    soundbuffer_writepos += SOUNDBUFFER_SIZE;
	}
    soundbuffer_len = soundbuffer_writepos - soundbuffer_readpos;

    if (soundbuffer_len < int2) PRINTF_DEBUG("soundbuffer_len < int2");

    return(0);
}

#endif /* use OSS, not USE_ALSA */

