/*
 * pitchtune - an instrument tuner
 * Copyright (C) 2000 Haakon Andre Hjortland
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * \file tuner.c
 * \brief The tuner part of the program
 *
 * The part which analyzes the input and draws on the widget
 * the graphs for the eye to detect pitch :)
 */



//*** Includes

#include <math.h>

#include "main.h"



/** 
 * \brief Inits sound card, sets default values for freq and samplepieces.
 */
void start_tuner()
{
    int int1;
    float float1;
    gboolean bool1;
  
    /* don't start if it's already started */
    if (tuner_started == TRUE) return;
  
    tuner_started=TRUE;
  
    /* init audio devs */
    int1 = sound_init();

    if (int1 != 0)
	{
            /* error initializing audio */
	    tuner_started=FALSE;
	    return;
	}
  
    // Set frequency
    float1=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_freq));
    set_frequency(float1);
  
    // Set number of samplepieces to display
    set_samplepieces_count_finetune(adjustment_finetune->value);
    set_samplepieces_count_coarsetune(adjustment_coarsetune->value);
  
    // Set up GTK timer
    if (timer_tag >= 0)
	PRINTF_DEBUG("Timer tag already set\n");
    timer_tag=gtk_timeout_add(20, (GtkFunction)timer_func, "");
    bool1=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_coarse_tuning));
    tuner_mode_coarse=FALSE;
    tuner_mode_coarse_new=bool1;
  
    // Reset sound buffer
    soundbuffer_len=0;
    soundbuffer_readpos=0;
    soundbuffer_writepos=0;
  
    return;
}



/** 
 * \brief Uninit sound card, kill GTK timer
 */
void stop_tuner()
{
    tuner_started=FALSE;

// Close sounddevice
    sound_deinit();

// Remove GTK timer
    if (timer_tag >= 0)
	{
	    gtk_timeout_remove(timer_tag);
	    timer_tag=-1;
	}

    return;
}



/**
 * \brief Reads audio buffers, and process it's data.
 * \param data Unused Gtk parameter
 * \todo Document the data parameter, document the timer_func function.
 */
gint timer_func (gpointer data)
{
    int sound_return_value;

    /* flush message queue */
    while (gtk_events_pending())
	{
	    gtk_main_iteration();
	}

    if (tuner_started==TRUE)
	{
	    // Read/write sound data, capture and playback of sine wave.
	    sound_return_value = sound_readwrite();

	    if (sound_return_value != 0)
		PRINTF_DEBUG("Error reading sound data, shutting off tuner\n");
    
	    if (tuner_mode_coarse != tuner_mode_coarse_new)
		{
		    tuner_mode_coarse = tuner_mode_coarse_new;
		    timer_count = 25;
		}
    
	    timer_count++;
    
	    if (
		((tuner_mode_coarse == TRUE) && (timer_count >= 25)) ||
		(tuner_mode_coarse == FALSE)
		)
		{
		    timer_count = 0;
		    if (tuner_mode_coarse_new == TRUE)
			process_data(PROCESS_DATA_REDO_SMALLSTEP, TRUE);
		    else
			process_data(PROCESS_DATA_REDO_NONE, FALSE);
		}
    
	    // If sound data reading failed, turn off tuner
	    if (sound_return_value != 0)
		{
		    stop_tuner();
		    return(FALSE);
		}
    
	    return(TRUE);
	}
    else
	{
	    timer_tag=-1;
	    return(FALSE); // Returning false kills timer
	}
} /* end timer_func() */



/*
 * Create drawing functions, include draw.c a couple of times!
 *
 */

#define DRAW_FUNC_BIG 1

#define DRAW_FUNC_DEPTH 8
#include "draw.c"
#undef DRAW_FUNC_DEPTH
#define DRAW_FUNC_DEPTH 16
#include "draw.c"
#undef DRAW_FUNC_DEPTH
#define DRAW_FUNC_DEPTH 32
#include "draw.c"

#undef DRAW_FUNC_BIG
#define DRAW_FUNC_BIG 0

#undef DRAW_FUNC_DEPTH
#define DRAW_FUNC_DEPTH 8
#include "draw.c"
#undef DRAW_FUNC_DEPTH
#define DRAW_FUNC_DEPTH 16
#include "draw.c"
#undef DRAW_FUNC_DEPTH
#define DRAW_FUNC_DEPTH 32
#include "draw.c"

#undef DRAW_FUNC_DEPTH
#undef DRAW_FUNC_BIG


/**
 * \brief Calls the draw_samplepieces_*() functions to draw audio on display
 *
 * This function will call the right draw_samplepieces_*() function to draw
 * according to the window size (big or not), and the visual depth.
 */
void draw_samplepieces(GdkWindow *window, int update)
{
    if (display_big)
	switch (gdkvisual1->depth)
	    {
	    case  8: draw_samplepieces_8_big();  break;
	    case 15: draw_samplepieces_16_big(); break;
	    case 16: draw_samplepieces_16_big(); break;
	    case 24: draw_samplepieces_32_big(); break;
	    case 32: draw_samplepieces_32_big(); break;
	    default: PRINTF_DEBUG("Unknown color depth (%i)\n", gdkvisual1->depth)
			 }
    else
	switch (gdkvisual1->depth)
	    {
	    case  8: draw_samplepieces_8_small();  break;
	    case 15: draw_samplepieces_16_small(); break;
	    case 16: draw_samplepieces_16_small(); break;
	    case 24: draw_samplepieces_32_small(); break;
	    case 32: draw_samplepieces_32_small(); break;
	    default: PRINTF_DEBUG("Unknown color depth (%i)\n", gdkvisual1->depth)
			 }

    return;
}



/**
 * \brief Process the audio data and draw it
 * \param redo Process all data again?
 * \param coarse_mode To determine which samplepieces_count to use
 * \todo Document process_data
 */
void process_data(enum process_data_redo_style redo, gboolean coarse_mode)
{
    double initial_sample_skip;
    int samples_skip;
    int samples_skip_last;
    int waves_skip;
    gboolean bool1;
    int int1;
    int int2;
    int int3;
    int int4;
    float float1;
    int which_samplepiece;
    int samplepieces_startpos;



    if (frequency_set == FALSE)
	{
	    PRINTF_DEBUG("Frequency information not set\n");
	    return;
	}

// Make sure variables have proper values
    if (backto_lastdraw_samplecount > 0.0)
        backto_lastdraw_samplecount = 0.0;

    if (samplepieces_last_valid >= SAMPLEPIECES_MAX)
	samplepieces_last_valid = SAMPLEPIECES_MAX - 1;

    if (coarse_mode == TRUE)
	samplepieces_count = samplepieces_count_coarsetune;
    else
	samplepieces_count = samplepieces_count_finetune;


// Find position in buffer at which to read data
    if (redo == PROCESS_DATA_REDO_NONE)
	{
	    bool1 = FALSE;
	    waves_skip = 0;
	    initial_sample_skip = soundbuffer_len + backto_lastdraw_samplecount;
    
	  label0:
	    samples_skip=(int)floor(initial_sample_skip +
				    wavelength_samples * waves_skip);
    
	    if (bool1 == FALSE)
		{
                    /* Scan upwards until too far out in the soundbuffer */
		    if (samples_skip + (int)(DRAWAREA_WIDTH * downsample_ratio) > soundbuffer_len)
			{
			    bool1 = TRUE;
			    waves_skip--;
			    goto label0;
			}
		    else
			{
			    waves_skip++;
			    goto label0;
			}
		}
	    else
		{
                    /* Scan downwards until not too far out in the soundbuffer */
		    if ((samples_skip + (int)(DRAWAREA_WIDTH * downsample_ratio) <= soundbuffer_len)
			&& (samples_skip >= 0))
			{
                            /* Found position */
			    goto label_pos_found;
			}
        
		    if (samples_skip >= 0)
			{
			    waves_skip--;
			    goto label0;
			}
		}
	}
    else
	{
            /* redo != PROCESS_DATA_REDO_NONE */
	    initial_sample_skip = soundbuffer_len - (int)(DRAWAREA_WIDTH * downsample_ratio);
	    if (initial_sample_skip >= 0)
		{
		    waves_skip = 0;
		    samples_skip = initial_sample_skip;
		    samplepieces_last_valid = -1;
		    goto label_pos_found;
		}
	}


    return;

/* Found position in soundbuffer at which to start reading */
  label_pos_found:

    which_samplepiece = 0;


    backto_lastdraw_samplecount += wavelength_samples * waves_skip;


/* Make room for next portion of wave data */
    if (redo == PROCESS_DATA_REDO_NONE)
	{
	    samplepieces_which_is_zero--;
	    samplepieces_which_is_zero += 10 * SAMPLEPIECES_MAX;
	    samplepieces_which_is_zero %= SAMPLEPIECES_MAX;
	}


/* Downsample next portion of data */
  label_downsample:

    samplepieces_last_valid += 1;

    samplepieces_startpos = ((samplepieces_which_is_zero + which_samplepiece)
			     % SAMPLEPIECES_MAX) * DRAWAREA_WIDTH;

    for (int1 = 0 ; int1 < DRAWAREA_WIDTH ; int1++)
	{
	    int4 = 0;
	    for (int2 = 0 ; int2 < ceil(downsample_ratio) ; int2++)
		{
		    int3 = (int)((double)soundbuffer_readpos + (double)samples_skip +
			    ((double)int1 * downsample_ratio) + (double)int2);

		    if (int3 < soundbuffer_readpos)
			PRINTF_DEBUG("Reading too low down in the buffer\n");
		    if (int3 > soundbuffer_writepos)
			PRINTF_DEBUG("Reading too high up in the buffer\n");

                    /* Sample value */
		    int4 += soundbuffer[int3 % SOUNDBUFFER_SIZE];
		}

            /* Div sample value by ratio ? */
            if (downsample_ratio >= 1)
                float1 = (double)int4 / (double)downsample_ratio;
            else
                float1 = (double)int4;

	    samplepieces_buffer[samplepieces_startpos + int1] = (int)floor(float1);
	}


// If redo , fill rest of samplepieces_buffer
    if ((redo != PROCESS_DATA_REDO_NONE) &&
	(which_samplepiece < samplepieces_count-1))
	{
	    samples_skip_last=samples_skip;
	    which_samplepiece++;
	    if (redo == PROCESS_DATA_REDO_BIGSTEP)
		{
		    // REDO_BIGSTEP
		    for (;;waves_skip--)
			{
			    samples_skip = (int)floor(initial_sample_skip + wavelength_samples * waves_skip);
			    if ((samples_skip + (int)(DRAWAREA_WIDTH * downsample_ratio)) <= samples_skip_last)
				{
				    if (samples_skip >= 0)
					// Downsample next portion of data
					goto label_downsample;
				    else
					goto label_draw;
				}
			}
		}
	    else
		{
		    // REDO_SMALLSTEP
		    waves_skip--;
		    samples_skip=(int)floor(
			initial_sample_skip + wavelength_samples*waves_skip);
		    if (samples_skip >= 0)
			{
			    // Downsample next portion of data
			    goto label_downsample;
			}
		}
	}


  label_draw:


    if (samplepieces_last_valid >= samplepieces_count)
	samplepieces_last_valid=samplepieces_count-1;


// Draw it
    draw_samplepieces(drawingarea1->window, TRUE);


    return;
}


